@@ -301,7 +301,6 @@ namespace xt
      ***************************************/
 
 
-
     /********************************
      * data_assigner implementation *
      ********************************/
@@ -366,7 +365,7 @@ namespace xt
         size_type size = e1.size();
         size_type simd_size = simd_type::size;
         size_type align_begin = is_aligned ? 0 : xsimd::get_alignment_offset(&(e1.data()), size, simd_size);
-        size_type align_end = align_begin + ((size - align_begin) &~(simd_size - 1));
+        size_type align_end = align_begin + ((size - align_begin) & ~(simd_size - 1));
         for (size_type i = 0; i < align_begin; ++i)
         {
             e1.data_element(i) = e2.data_element(i);
@@ -393,7 +392,7 @@ namespace xt
         inline void trivial_assigner_run_impl(E1&, const E2&, std::false_type)
         {
             XTENSOR_PRECONDITION(false,
-                "Internal error: trivial_assigner called with unequal types.");
+                "Internal error: trivial_assigner called with unrelated types.");
         }
     }
 
