@@ -11,10 +11,11 @@
 
 #include <algorithm>
 #include <functional>
+#include <iostream>
 #include <numeric>
 #include <stdexcept>
-#include <iostream>
 
+#include "xtl/xmeta_utils.hpp"
 #include "xtl/xsequence.hpp"
 
 #include "xiterable.hpp"
@@ -103,9 +104,16 @@ namespace xt
         template <class... Args>
         const_reference at(Args... args) const;
 
-        reference operator[](const xindex& index);
+        template <class S>
+        disable_integral_t<S, reference> operator[](const S& index);
+        template <class I>
+        reference operator[](std::initializer_list<I> index);
         reference operator[](size_type i);
-        const_reference operator[](const xindex& index) const;
+
+        template <class S>
+        disable_integral_t<S, const_reference> operator[](const S& index) const;
+        template <class I>
+        const_reference operator[](std::initializer_list<I> index) const;
         const_reference operator[](size_type i) const;
 
         template <class It>
@@ -559,11 +567,20 @@ namespace xt
      * than the number of dimensions of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::operator[](const xindex& index) -> reference
+    template <class S>
+    inline auto xcontainer<D>::operator[](const S& index)
+        -> disable_integral_t<S, reference>
     {
         return element(index.cbegin(), index.cend());
     }
 
+    template <class D>
+    template <class I>
+    inline auto xcontainer<D>::operator[](std::initializer_list<I> index) -> reference
+    {
+        return element(index.begin(), index.end());
+    }
+
     template <class D>
     inline auto xcontainer<D>::operator[](size_type i) -> reference
     {
@@ -577,11 +594,20 @@ namespace xt
      * than the number of dimensions of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::operator[](const xindex& index) const -> const_reference
+    template <class S>
+    inline auto xcontainer<D>::operator[](const S& index) const
+        -> disable_integral_t<S, const_reference>
     {
         return element(index.cbegin(), index.cend());
     }
 
+    template <class D>
+    template <class I>
+    inline auto xcontainer<D>::operator[](std::initializer_list<I> index) const -> const_reference
+    {
+        return element(index.begin(), index.end());
+    }
+
     template <class D>
     inline auto xcontainer<D>::operator[](size_type i) const -> const_reference
     {
@@ -700,7 +726,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::begin() noexcept -> select_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_begin<L>();
         }, /*else*/ [&](auto self)
@@ -713,7 +739,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::end() noexcept -> select_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_end<L>();
         }, /*else*/ [&](auto self)
@@ -740,7 +766,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::cbegin() const noexcept -> select_const_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_cbegin<L>();
         }, /*else*/ [&](auto self)
@@ -753,7 +779,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::cend() const noexcept -> select_const_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_cend<L>();
         }, /*else*/ [&](auto self)
@@ -766,7 +792,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::rbegin() noexcept -> select_reverse_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_rbegin<L>();
         }, /*else*/ [&](auto self)
@@ -779,7 +805,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::rend() noexcept -> select_reverse_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_rend<L>();
         }, /*else*/ [&](auto self)
@@ -806,7 +832,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::crbegin() const noexcept -> select_const_reverse_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_crbegin<L>();
         }, /*else*/ [&](auto self)
@@ -819,7 +845,7 @@ namespace xt
     template <layout_type L>
     inline auto xcontainer<D>::crend() const noexcept -> select_const_reverse_iterator<L>
     {
-        return static_if<L == static_layout>([&](auto self)
+        return xtl::mpl::static_if<L == static_layout>([&](auto self)
         {
             return self(*this).template storage_crend<L>();
         }, /*else*/ [&](auto self)
