@@ -193,7 +193,7 @@ namespace xt
         using reverse_iterator = typename iterable_base::reverse_iterator;
         using const_reverse_iterator = typename iterable_base::const_reverse_iterator;
 
-        template <class Func>
+        template <class Func, class U = std::enable_if<!std::is_base_of<Func, self_type>::value>>
         xfunction(Func&& f, CT... e) noexcept;
 
         size_type size() const noexcept;
@@ -207,7 +207,10 @@ namespace xt
         template <class... Args>
         const_reference at(Args... args) const;
 
-        const_reference operator[](const xindex& index) const;
+        template <class S>
+        disable_integral_t<S, const_reference> operator[](const S& index) const;
+        template <class I>
+        const_reference operator[](std::initializer_list<I> index) const;
         const_reference operator[](size_type i) const;
 
         template <class It>
@@ -448,7 +451,7 @@ namespace xt
      * @param e the \ref xexpression arguments
      */
     template <class F, class R, class... CT>
-    template <class Func>
+    template <class Func, class U>
     inline xfunction<F, R, CT...>::xfunction(Func&& f, CT... e) noexcept
         : m_e(e...), m_f(std::forward<Func>(f)), m_shape(xtl::make_sequence<shape_type>(0, size_type(1))),
           m_shape_computed(false)
@@ -540,11 +543,20 @@ namespace xt
     }
 
     template <class F, class R, class... CT>
-    inline auto xfunction<F, R, CT...>::operator[](const xindex& index) const -> const_reference
+    template <class S>
+    inline auto xfunction<F, R, CT...>::operator[](const S& index) const
+        -> disable_integral_t<S, const_reference>
     {
         return element(index.cbegin(), index.cend());
     }
 
+    template <class F, class R, class... CT>
+    template <class I>
+    inline auto xfunction<F, R, CT...>::operator[](std::initializer_list<I> index) const -> const_reference
+    {
+        return element(index.begin(), index.end());
+    }
+
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::operator[](size_type i) const -> const_reference
     {
