@@ -6,8 +6,8 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XINDEXVIEW_HPP
-#define XINDEXVIEW_HPP
+#ifndef XTENSOR_INDEXVIEW_HPP
+#define XTENSOR_INDEXVIEW_HPP
 
 #include <algorithm>
 #include <cstddef>
@@ -39,10 +39,6 @@ namespace xt
         using inner_shape_type = std::array<std::size_t, 1>;
         using const_stepper = xindexed_stepper<xindexview<CT, I>>;
         using stepper = xindexed_stepper<xindexview<CT, I>, false>;
-        using const_iterator = xiterator<const_stepper, inner_shape_type*, DEFAULT_LAYOUT>;
-        using iterator = xiterator<stepper, inner_shape_type*, DEFAULT_LAYOUT>;
-        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
-        using reverse_iterator = std::reverse_iterator<iterator>;
     };
 
     /**************
@@ -65,7 +61,7 @@ namespace xt
      */
     template <class CT, class I>
     class xindexview : public xview_semantic<xindexview<CT, I>>,
-                       public xexpression_iterable<xindexview<CT, I>>
+                       public xiterable<xindexview<CT, I>>
     {
     public:
 
@@ -81,7 +77,7 @@ namespace xt
         using size_type = typename xexpression_type::size_type;
         using difference_type = typename xexpression_type::difference_type;
 
-        using iterable_base = xexpression_iterable<self_type>;
+        using iterable_base = xiterable<self_type>;
         using inner_shape_type = typename iterable_base::inner_shape_type;
         using shape_type = inner_shape_type;
         using strides_type = shape_type;
@@ -113,8 +109,11 @@ namespace xt
 
         reference operator()();
         template <class... Args>
-        reference operator()(std::size_t idx, Args... /*args*/);
-        reference operator[](const xindex& index);
+        reference operator()(size_type idx, Args... /*args*/);
+        template <class S>
+        disable_integral_t<S, reference> operator[](const S& index);
+        template <class OI>
+        reference operator[](std::initializer_list<OI> index);
         reference operator[](size_type i);
 
         template <class It>
@@ -122,8 +121,11 @@ namespace xt
 
         const_reference operator()() const;
         template <class... Args>
-        const_reference operator()(std::size_t idx, Args... /*args*/) const;
-        const_reference operator[](const xindex& index) const;
+        const_reference operator()(size_type idx, Args... /*args*/) const;
+        template <class S>
+        disable_integral_t<S, const_reference> operator[](const S& index) const;
+        template <class OI>
+        const_reference operator[](std::initializer_list<OI> index) const;
         const_reference operator[](size_type i) const;
 
         template <class It>
@@ -262,7 +264,7 @@ namespace xt
     template <class CT, class I>
     inline void xindexview<CT, I>::assign_temporary_impl(temporary_type&& tmp)
     {
-        std::copy(tmp.cbegin(), tmp.cend(), this->xbegin());
+        std::copy(tmp.cbegin(), tmp.cend(), this->begin());
     }
 
     /**
@@ -321,7 +323,7 @@ namespace xt
 
     template <class CT, class I>
     template <class... Args>
-    inline auto xindexview<CT, I>::operator()(std::size_t idx, Args... /*args*/) -> reference
+    inline auto xindexview<CT, I>::operator()(size_type idx, Args... /*args*/) -> reference
     {
         return m_e[m_indices[idx]];
     }
@@ -333,17 +335,27 @@ namespace xt
      */
     template <class CT, class I>
     template <class... Args>
-    inline auto xindexview<CT, I>::operator()(std::size_t idx, Args... /*args*/) const -> const_reference
+    inline auto xindexview<CT, I>::operator()(size_type idx, Args... /*args*/) const -> const_reference
     {
         return m_e[m_indices[idx]];
     }
 
     template <class CT, class I>
-    inline auto xindexview<CT, I>::operator[](const xindex& index) -> reference
+    template <class S>
+    inline auto xindexview<CT, I>::operator[](const S& index)
+        -> disable_integral_t<S, reference>
     {
         return m_e[m_indices[index[0]]];
     }
 
+    template <class CT, class I>
+    template <class OI>
+    inline auto xindexview<CT, I>::operator[](std::initializer_list<OI> index)
+        -> reference
+    {
+        return m_e[m_indices[*(index.begin())]];
+    }
+
     template <class CT, class I>
     inline auto xindexview<CT, I>::operator[](size_type i) -> reference
     {
@@ -351,11 +363,21 @@ namespace xt
     }
 
     template <class CT, class I>
-    inline auto xindexview<CT, I>::operator[](const xindex& index) const -> const_reference
+    template <class S>
+    inline auto xindexview<CT, I>::operator[](const S& index) const
+        -> disable_integral_t<S, const_reference>
     {
         return m_e[m_indices[index[0]]];
     }
 
+    template <class CT, class I>
+    template <class OI>
+    inline auto xindexview<CT, I>::operator[](std::initializer_list<OI> index) const
+        -> const_reference
+    {
+        return m_e[m_indices[*(index.begin())]];
+    }
+
     template <class CT, class I>
     inline auto xindexview<CT, I>::operator[](size_type i) const -> const_reference
     {
@@ -365,7 +387,7 @@ namespace xt
     /**
      * Returns a reference to the element at the specified position in the xindexview.
      * @param first iterator starting the sequence of indices
-     * The number of indices in the squence should be equal to or greater 1.
+     * The number of indices in the sequence should be equal to or greater 1.
      */
     template <class CT, class I>
     template <class It>
