@@ -279,7 +279,7 @@ namespace xt
         // lead to ambiguous call when the function parameter is xexpression<E>
         // on gcc: the compiler can't choose between xexpression<xoptional_function>
         // and xexpression<xfunction>.
-        
+
         using only_scalar = typename implementation_type::only_scalar;
         using functor_type = typename implementation_type::functor_type;
         using value_type = typename implementation_type::value_type;
@@ -317,7 +317,7 @@ namespace xt
         using reverse_broadcast_iterator = typename iterable_base::template reverse_broadcast_iterator<S, L>;
         template <class S, layout_type L>
         using const_reverse_broadcast_iterator = typename iterable_base::template const_reverse_broadcast_iterator<S, L>;
-        
+
         using storage_iterator = typename implementation_type::storage_iterator;
         using const_storage_iterator = typename implementation_type::const_storage_iterator;
         using reverse_storage_iterator = typename implementation_type::reverse_storage_iterator;
@@ -327,7 +327,7 @@ namespace xt
         using const_iterator = typename iterable_base::const_iterator;
         using reverse_iterator = typename iterable_base::reverse_iterator;
         using const_reverse_iterator = typename iterable_base::const_reverse_iterator;
-        
+
         template <class Func>
         xoptional_function(Func&& func, CT... e) noexcept;
 
@@ -342,7 +342,10 @@ namespace xt
         template <class... Args>
         const_reference at(Args... args) const;
 
-        const_reference operator[](const xindex& index) const;
+        template <class S>
+        disable_integral_t<S, const_reference> operator[](const S& index) const;
+        template <class I>
+        const_reference operator[](std::initializer_list<I> index) const;
         const_reference operator[](size_type i) const;
 
         template <class It>
@@ -393,7 +396,7 @@ namespace xt
 
         template <class align, class simd = simd_value_type>
         detail::simd_return_type_t<functor_type, simd> load_simd(size_type i) const;
-        
+
         // Specific to xoptional_function
 
         using expression_tag = xoptional_expression_tag;
@@ -414,7 +417,7 @@ namespace xt
     private:
 
         implementation_type m_func;
-        
+
         template <std::size_t... I>
         value_expression value_impl(std::index_sequence<I...>) const;
 
@@ -474,7 +477,17 @@ namespace xt
     }
 
     template <class F, class R, class... CT>
-    inline auto xoptional_function<F, R, CT...>::operator[](const xindex& index) const -> const_reference
+    template <class S>
+    inline auto xoptional_function<F, R, CT...>::operator[](const S& index) const
+        -> disable_integral_t<S, const_reference>
+    {
+        return m_func[index];
+    }
+
+    template <class F, class R, class... CT>
+    template <class I>
+    inline auto xoptional_function<F, R, CT...>::operator[](std::initializer_list<I> index) const
+        -> const_reference
     {
         return m_func[index];
     }
@@ -573,7 +586,7 @@ namespace xt
 
     template <class F, class R, class... CT>
     template <class S>
-    inline auto xoptional_function<F, R, CT...>::stepper_end(const S& shape, layout_type l) const noexcept  -> const_stepper
+    inline auto xoptional_function<F, R, CT...>::stepper_end(const S& shape, layout_type l) const noexcept -> const_stepper
     {
         return m_func.stepper_end(shape, l);
     }
