@@ -125,7 +125,10 @@ namespace xt
         reference operator()(Args... args);
         template <class... Args>
         reference at(Args... args);
-        reference operator[](const xindex& index);
+        template <class OS>
+        disable_integral_t<OS, reference> operator[](const OS& index);
+        template <class I>
+        reference operator[](std::initializer_list<I> index);
         reference operator[](size_type i);
         template <class It>
         reference element(It first, It last);
@@ -134,7 +137,10 @@ namespace xt
         const_reference operator()(Args... args) const;
         template <class... Args>
         const_reference at(Args... args) const;
-        const_reference operator[](const xindex& index) const;
+        template <class OS>
+        disable_integral_t<OS, const_reference> operator[](const OS& index) const;
+        template <class I>
+        const_reference operator[](std::initializer_list<I> index) const;
         const_reference operator[](size_type i) const;
         template <class It>
         const_reference element(It first, It last) const;
@@ -178,8 +184,8 @@ namespace xt
         size_type underlying_size(size_type dim) const;
 
         xtl::xclosure_pointer<self_type&> operator&() &;
-        xtl::xclosure_pointer<const self_type&> operator&() const&;
-        xtl::xclosure_pointer<self_type> operator&() && ;
+        xtl::xclosure_pointer<const self_type&> operator&() const &;
+        xtl::xclosure_pointer<self_type> operator&() &&;
 
     private:
 
@@ -377,7 +383,7 @@ namespace xt
     inline auto xview<CT, S...>::operator=(const xexpression<E>& e) -> self_type&
     {
         bool cond = (e.derived_cast().shape().size() == dimension()) &&
-                    std::equal(shape().begin(), shape().end(), e.derived_cast().shape().begin());
+            std::equal(shape().begin(), shape().end(), e.derived_cast().shape().begin());
         if (!cond)
         {
             semantic_base::operator=(broadcast(e.derived_cast(), shape()));
@@ -488,11 +494,21 @@ namespace xt
     }
 
     template <class CT, class... S>
-    inline auto xview<CT, S...>::operator[](const xindex& index) -> reference
+    template <class OS>
+    inline auto xview<CT, S...>::operator[](const OS& index)
+        -> disable_integral_t<OS, reference>
     {
         return element(index.cbegin(), index.cend());
     }
 
+    template <class CT, class... S>
+    template <class I>
+    inline auto xview<CT, S...>::operator[](std::initializer_list<I> index)
+        -> reference
+    {
+        return element(index.begin(), index.end());
+    }
+
     template <class CT, class... S>
     inline auto xview<CT, S...>::operator[](size_type i) -> reference
     {
@@ -544,11 +560,21 @@ namespace xt
     }
 
     template <class CT, class... S>
-    inline auto xview<CT, S...>::operator[](const xindex& index) const -> const_reference
+    template <class OS>
+    inline auto xview<CT, S...>::operator[](const OS& index) const
+        -> disable_integral_t<OS, const_reference>
     {
         return element(index.cbegin(), index.cend());
     }
 
+    template <class CT, class... S>
+    template <class I>
+    inline auto xview<CT, S...>::operator[](std::initializer_list<I> index) const
+        -> const_reference
+    {
+        return element(index.begin(), index.end());
+    }
+
     template <class CT, class... S>
     inline auto xview<CT, S...>::operator[](size_type i) const -> const_reference
     {
@@ -660,13 +686,13 @@ namespace xt
     }
 
     template <class CT, class... S>
-    inline auto xview<CT, S...>::operator&() const& -> xtl::xclosure_pointer<const self_type&>
+    inline auto xview<CT, S...>::operator&() const & -> xtl::xclosure_pointer<const self_type&>
     {
         return xtl::closure_pointer(*this);
     }
 
     template <class CT, class... S>
-    inline auto xview<CT, S...>::operator&() && ->xtl::xclosure_pointer<self_type>
+    inline auto xview<CT, S...>::operator&() && -> xtl::xclosure_pointer<self_type>
     {
         return xtl::closure_pointer(std::move(*this));
     }
@@ -756,12 +782,10 @@ namespace xt
     inline auto xview<CT, S...>::make_index(It first, It last) const -> base_index_type
     {
         auto index = xtl::make_sequence<typename xexpression_type::shape_type>(m_e.dimension(), 0);
-        auto func1 = [&first](const auto& s)
-        {
+        auto func1 = [&first](const auto& s) {
             return get_slice_value(s, first);
         };
-        auto func2 = [](const auto& s)
-        {
+        auto func2 = [](const auto& s) {
             return xt::value(s, 0);
         };
         for (size_type i = 0; i != m_e.dimension(); ++i)
